@@ -52,7 +52,7 @@ property rows => (
 	value => [],
 );
 
-function total_rows => sub {
+function count => sub {
 	return scalar @{$_[0]->rows};
 };
 
@@ -195,19 +195,114 @@ function delete_row_col => sub {
 };
 
 function sort => sub {
-	my ($self, $col, $order) = @_;
+	my ($self, $col, $order, $return) = @_;
+	$col = $self->find_column_index($col);
 	my @rows = $order eq 'asc'
 		? sort { $a->get_col($col)->value cmp $b->get_col($col)->value } @{$self->rows}
 		: sort { $b->get_col($col)->value cmp $a->get_col($col)->value } @{$self->rows};
-	$self->rows = \@rows;
+	$self->rows = \@rows unless $return;
+	return \@rows;
 };
 
 function search => sub {
 	my ($self, $col, $search) = @_;
+	$col = $self->find_column_index($col);
+	my ($i, @indexes) = (0);
 	my @rows = grep {
-		$_->get_col($col)->value =~ m/$search/i
+		if ( $_->get_col($col)->value =~ m/$search/i ) {
+			push @indexes, $i++;
+			return $_;
+		}
+		$i++;
+		return ();
 	} @{$self->rows};
-	return \@rows;
+	return (\@rows, \@indexes);
+};
+
+function find => sub {
+	my ($self, $col, $search) = @_;
+	$col = $self->find_column_index($col);
+	my ($i, $found) = (0, undef);
+	for ( @{$self->rows} ) {
+		if ($_->get_col($col)->value =~ m/$search/i) {
+			$found = $i;
+			last;
+		}
+		$i++;
+	}
+	return $found;
+};
+
+function find_column_index => sub {
+	my ($self, $col) = @_;
+	if ($col !~ m/^\d+$/) {
+		for (@{$self->headers}) {
+			if ($_->name =~ m/^($col)$/) {
+				$col = $_->index;
+			}
+		}
+	}
+	return $col;
+};
+
+function sum => sub {
+	my ($self, $col) = @_;
+	$col = $self->find_column_index($col);
+	my $sum = 0;
+	for (@{$self->rows}) {
+		my $c = $_->get_col($col);
+		if ($c->value !~ m/^\d+$/) {
+			die "Cannot sum column as it has non numeric values";
+		}
+		$sum += $c->value;
+	}
+	return $sum;
+};
+
+function mean => sub {
+	my ($self, $col) = @_;
+	my $sum = $self->sum($col);
+	return $sum / scalar @{$self->rows};
+};
+
+function median => sub {
+	my ($self, $col, $as_row) = @_;
+	$col = $self->find_column_index($col);
+	my $rows = $self->sort($col, 'asc', 1);
+	my $median = int(scalar @{$rows} / 2);
+	if ($median % 2 != 0) {
+		$median += 1;
+	}
+	return $as_row ? $rows->[$median - 1] : $rows->[$median - 1]->get_col($col)->value;
+};
+
+function mode => sub {
+	my ($self, $col) = @_;
+	$col = $self->find_column_index($col);
+	my %map;
+	$map{$_->get_col($col)->value}++ for (@{$self->rows});
+	my ($key, $mode) = ('', 0);
+	for my $k (keys %map) {
+		if ($map{$k} > $mode) {
+			$key = $k;
+			$mode = $map{$k};
+		}
+	}
+	return ($key, $mode);
+};
+
+function min => sub {
+	my ($self, $col, $as_row) = @_;
+	$col = $self->find_column_index($col);
+	my $rows = $self->sort($col, 'asc', 1);
+	return $as_row ? $rows->[0] : $rows->[0]->get_col($col)->value;
+};
+
+function max => sub {
+	my ($self, $col, $as_row) = @_;
+	$col = $self->find_column_index($col);
+	my $rows = $self->sort($col, 'desc', 1);
+	return $as_row ? $rows->[0] : $rows->[0]->get_col($col)->value;
 };
 
 function headers_as_array => sub {
